
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.event.MouseAdapter;
import java.awt.event.InputEvent;

// A controller class which controls the 
// drawing process and maintains its own
// painting mode and state
public class StrokeListener
	extends MouseAdapter implements MouseMotionListener, MouseWheelListener {

	// Our buffered bitmap
	private GraphicsModel model;

	// Coordinates of previous mouse drag event
	private int oldX, oldY;
	// If this mouse click is part of a drag,
	// stayedStill == false
	private boolean stayedStill;

	// The active painting mode
	private boolean zoomMode;
	private PaintMode paintMode;
	private static enum PaintMode {LAME, RANDOM};

	// Constants giving the scaling factors
	// for zooming in, zooming out, and
	// the special constant zoom mode
	private static double SCALE_IN = 1.25;
	private static double SCALE_OUT = 0.9;
	private static double ZOOM_FACTOR = 1.17;
	
	public StrokeListener(GraphicsModel model) {
		this.model = model;
		stayedStill = true;
		zoomMode = false;
		paintMode = PaintMode.LAME;
	}
	
	// Mouse wheel can be used for manual zooming.
	// The zoom is centered on the cursor position.
	public void mouseWheelMoved(MouseWheelEvent we) {
		if (we.getWheelRotation() > 0) {
			model.zoom(SCALE_IN, we.getX(), we.getY());
		}
		else {
			model.zoom(SCALE_OUT, we.getX(), we.getY());
		}
	}
	
	// Painting is done by dragging
	public void mouseDragged(MouseEvent mevent) {
		int modifiers = mevent.getModifiersEx();

		// Dragging with left mouse button only
		if ((modifiers & InputEvent.BUTTON1_DOWN_MASK) == InputEvent.BUTTON1_DOWN_MASK) {

			// In lame mode, a line is drawn between the previous
			// cursor position and the next one
			if (paintMode == PaintMode.LAME ) {
				// The next click will be part of this drag
				stayedStill = false;

				if (oldX != mevent.getX() || oldY != mevent.getY()) {
					model.paintLine(oldX, oldY, mevent.getX(), mevent.getY());
				}
				// Store current coordinates for next line
				oldX = mevent.getX();
				oldY = mevent.getY();
			}
			// In random mode, a randomly colored dot is drawn
			else if (paintMode == PaintMode.RANDOM) {
				paintDot(mevent.getX(), mevent.getY());
			}

			// A special effect when in zoom mode
			if (zoomMode) {
				model.zoom(ZOOM_FACTOR, mevent.getX(), mevent.getY());
			}
		}
	}

	// Store current coordinates for next line
	public void mousePressed(MouseEvent mevent) {
		if (mevent.getButton() == MouseEvent.BUTTON1) {
			oldX = mevent.getX();
			oldY = mevent.getY();
		}
	}
	
	public void mouseClicked(MouseEvent mevent) {
		switch (mevent.getButton()) {
		case (MouseEvent.BUTTON1) :
			if (paintMode == PaintMode.LAME) {
				// this represents a click with no drag
				if (stayedStill) {
					paintDot(mevent.getX(), mevent.getY());
				}
				else {
					// reset the dragging state after drag
					stayedStill = true;
				}
			}
			else if (paintMode == PaintMode.RANDOM) {
				// just draw a dot
				paintDot(mevent.getX(), mevent.getY());
			}
			break;
		// Middle button switches zoom mode
		case (MouseEvent.BUTTON2) :
			switchZoom();
			break;
		// Right button changes paint mode 
		case (MouseEvent.BUTTON3) :
			nextMode();
			break;
		default:
			break;
		}
	}
	
	// Draws a dot, with default or random color
	// depending on mode
	private void paintDot(int x, int y) {
		if (paintMode == PaintMode.LAME) {
			model.paintDot(x, y);
		}
		else if (paintMode == PaintMode.RANDOM) {
			model.paintRandomColoredDot(x, y);
		}
	}
	
	private boolean switchZoom() {
		return (zoomMode = !zoomMode);
	}
	
	private PaintMode nextMode() {
		if (paintMode == PaintMode.LAME) {
			paintMode = PaintMode.RANDOM;
		}
		else {
			paintMode = PaintMode.LAME;
		}
		return paintMode;
	}

	public void mouseMoved(MouseEvent me) { }
	
}
